/******************************************
 * REAL-TIME SLIT-SCAN                    *
 * Tutor: Paolo Pasteris                  *
 * progettoMM 11/12 Unimi                 *
 ******************************************/

import codeanticode.gsvideo.*;

GSCapture cam;
PImage[] buffer;
PImage img;
GSMovieMaker mm;
int index,fps,direction,speed;

//PFont font;

void setup() {
  fps = 25;
  size(800,600);
  background(0);
  frameRate(fps);
  speed = 1;
  // camera and buffer initialization
  cam = new GSCapture(this, 320, 240, "/dev/video0");
  cam.start();
  buffer = new PImage[max(cam.width,cam.height)];
  for (int i = 0; i < buffer.length; i++) {
    buffer[i] = new PImage(cam.width,cam.height);
  }
  img = new PImage(cam.width,cam.height);
  // video encoder initialization
  mm = new GSMovieMaker(this, width, height, "slitscan.ogg",
                        GSMovieMaker.THEORA, GSMovieMaker.HIGH,
                        fps);
  index = 0;
  direction = 0;
  
}

void draw() {
  // read frame from camera
  if (cam.available() == true) {
    cam.read();
  }
  // put it in the buffer
  for (int i = 0; i < speed; i++) {
    int x = (index+i) % buffer.length;
    buffer[x].copy(cam,0,0,cam.width,cam.height,0,0,cam.width,cam.height);
  }
  // update the slit-scan image
  switch (direction) {
    case 0:
      topDown();
      break;
    case 1:
      bottomUp();
      break;
    case 2:
      leftRight();
      break;
    case 3:
      rightLeft();
      break;
  }
  
  // display the slit scan on screen
  image(img, 0, 0, width, height);
  
  // add the frame to the video, if recording
  if (mm.isRecording()) mm.addFrame(img.pixels);
  
  // print the instructions on screen
  //fill(255,255,0);
  text("ARROWS: slit-scan direction\n" +
       "SPACE: start/stop recording\n"+
       "SPEED: +/-\n", 10, 16);
  if (mm.isRecording()) text("RECORDING", 10, height - 6);
}

// processing algorithms
void topDown() {
  int i = index;
  for (int y = cam.height-1; y >= 0; y--) {
    i = (i + 1) % cam.height;
    for (int x = 0; x < cam.width; x++) {
      img.set(x,y,buffer[i].get(x,y));
    }
  }
  index = (index + speed) % cam.height;
}

void bottomUp() {
  int i = index;
  for (int y = 0; y < cam.height; y++) {
    i = (i + 1) % cam.height;
    for (int x = 0; x < cam.width; x++) {
      img.set(x,y,buffer[i].get(x,y));
    }
  }
  index = (index + speed) % cam.height;
}

void leftRight() {
  int i = index;
  for (int x = cam.width-1; x >= 0; x--) {
    i = (i + 1) % cam.width;
    for (int y = 0; y < cam.height; y++) {
      img.set(x,y,buffer[i].get(x,y));
    }
  }
  index = (index + speed) % cam.width;
}

void rightLeft() {
  int i = index;
  for (int x = 0; x < cam.width; x++) {
    i = (i + 1) % cam.width;
    for (int y = 0; y < cam.height; y++) {
      img.set(x,y,buffer[i].get(x,y));
    }
  }
  index = (index + speed) % cam.width;
}

void keyPressed() {
  // select slit-scan direction
  if (key == CODED) {
    if (keyCode == DOWN) {
      direction = 0;
      return;
    }
    if (keyCode == UP) {
      direction = 1;
      return;
    }
    if (keyCode == RIGHT) {
      direction = 2;
      return;
    }
    if (keyCode == LEFT) {
      direction = 3;
      return;
    } 
  }
  // toggles the recording
  if (key == ' ') {
    if (mm.isRecording()) mm.finish();
    else mm.start();
    return;
  }
  if (key == '+') {
    speed = speed >= buffer.length ? buffer.length-1 : speed + 1;
    index = 0;
    return;
  }
  if (key == '-') {
    speed = speed <= 1 ? 1 : speed - 1;
    index = 0;
    return;
  }
}
